@@ -24,7 +24,11 @@
  * outside of the core library.
  */
 
+#ifdef USE_SDL3
+#include <SDL3/SDL.h>
+#else
 #include <SDL.h>
+#endif 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
