@@ -59,6 +59,9 @@ MainDialog::MainDialog(QWidget* parent, Thread::SDLThread* sdlThread, bool romCo
             &MainDialog::on_ControllerWidget_UserProfileRemoved);
     }
 
+    // initialize input profile DB
+    this->inputProfileDB.Load();
+
     // always add keyboard device
     for (auto& controllerWidget : this->controllerWidgets)
     {
@@ -68,14 +71,14 @@ MainDialog::MainDialog(QWidget* parent, Thread::SDLThread* sdlThread, bool romCo
         // so we only have to expose it there
         if (controllerWidget == this->controllerWidgets.last())
         {
-            controllerWidget->AddInputDevice({ InputDeviceType::EmulateVRU, "Voice Recognition Unit" });
+            controllerWidget->AddInputDevice({ InputDeviceType::EmulateVRU, "Voice Recognition Unit" }, {});
         }
 #endif // VRU
         controllerWidget->SetAllowKeyboardForAutomatic(controllerWidget == this->controllerWidgets.first());
 
-        controllerWidget->AddInputDevice({ InputDeviceType::None, "None" });
-        controllerWidget->AddInputDevice({ InputDeviceType::Automatic, "Automatic" });
-        controllerWidget->AddInputDevice({ InputDeviceType::Keyboard, "Keyboard" });
+        this->addInputDevice(controllerWidget, { InputDeviceType::None, "None" });
+        this->addInputDevice(controllerWidget, { InputDeviceType::Automatic, "Automatic" });
+        this->addInputDevice(controllerWidget, { InputDeviceType::Keyboard, "Keyboard" });
         controllerWidget->SetInitialized(true);
     }
 
@@ -92,11 +95,24 @@ MainDialog::~MainDialog()
     this->closeInputDevice();
 }
 
+void MainDialog::addInputDevice(Widget::ControllerWidget* controllerWidget, const InputDevice& device)
+{
+    // only fallback to fallback profile when
+    // we have a controller device
+    InputProfileDBEntry inputProfile = this->inputProfileDB.FindEntry(device,
+                                            device.type == InputDeviceType::Automatic ||
+                                            device.type == InputDeviceType::Joystick);
+
+    controllerWidget->AddInputDevice(device, inputProfile);
+}
+
 void MainDialog::addInputDevice(const InputDevice& device)
 {
+    InputProfileDBEntry inputProfile = this->inputProfileDB.FindEntry(device);
+
     for (auto& controllerWidget : this->controllerWidgets)
     {
-        controllerWidget->AddInputDevice(device);
+        controllerWidget->AddInputDevice(device, inputProfile);
     }
 }
 
@@ -112,14 +128,14 @@ void MainDialog::openInputDevice(InputDevice device)
 {
     Widget::ControllerWidget* controllerWidget;
     controllerWidget = this->controllerWidgets.at(this->tabWidget->currentIndex());
+    InputProfileDBEntry inputProfile = {};
 
     // we don't need to open a non-joystick device
     if (device.type != InputDeviceType::Automatic &&
         device.type != InputDeviceType::Joystick)
     {
         this->currentDevice = { };
-        controllerWidget->SetCurrentJoystickID(this->currentDevice.id);
-        controllerWidget->SetCurrentJoystick(nullptr, nullptr);
+        controllerWidget->SetCurrentInputDevice(this->currentDevice.id, nullptr, nullptr, inputProfile);
         return;
     }
 
@@ -130,12 +146,12 @@ void MainDialog::openInputDevice(InputDevice device)
         if (currentIndex < this->inputDeviceList.size())
         { // use device when there's one
             device.id = this->inputDeviceList.at(currentIndex).id;
+            inputProfile = this->inputProfileDB.FindEntry(this->inputDeviceList.at(currentIndex));
         }
         else
         { // no device found, fallback to keyboard
             this->currentDevice = { InputDeviceType::Keyboard, "Keyboard" };
-            controllerWidget->SetCurrentJoystickID(this->currentDevice.id);
-            controllerWidget->SetCurrentJoystick(nullptr, nullptr);
+            controllerWidget->SetCurrentInputDevice(this->currentDevice.id, nullptr, nullptr, inputProfile);
             return;
         }
     }
@@ -145,18 +161,16 @@ void MainDialog::openInputDevice(InputDevice device)
         (controllerMode == 2))
     {
         this->currentJoystick = nullptr;
-        this->currentController = SDL_OpenGamepad(device.id);
+        this->currentGamepad = SDL_OpenGamepad(device.id);
     }
     else if (controllerMode == 0 || controllerMode == 1)
     {
         this->currentJoystick = SDL_OpenJoystick(device.id);
-        this->currentController = nullptr;
+        this->currentGamepad = nullptr;
     }
 
     this->currentDevice = device;
-    controllerWidget->SetCurrentJoystickID(device.id);
-    controllerWidget->SetIsCurrentJoystickGameController(currentController != nullptr);
-    controllerWidget->SetCurrentJoystick(this->currentJoystick, this->currentController);
+    controllerWidget->SetCurrentInputDevice(device.id, this->currentJoystick, this->currentGamepad, inputProfile);
 }
 
 void MainDialog::closeInputDevice()
@@ -167,10 +181,10 @@ void MainDialog::closeInputDevice()
         this->currentJoystick = nullptr;
     }
 
-    if (this->currentController != nullptr)
+    if (this->currentGamepad != nullptr)
     {
-        SDL_CloseGamepad(this->currentController);
-        this->currentController = nullptr;
+        SDL_CloseGamepad(this->currentGamepad);
+        this->currentGamepad = nullptr;
     }
 }
 
@@ -202,15 +216,6 @@ void MainDialog::on_InputPollTimer_triggered()
             break;
     }
 
-    // check if controller has been disconnected,
-    // if so, keep trying to re-open it
-    if ((this->currentJoystick != nullptr && !SDL_JoystickConnected(this->currentJoystick)) ||
-        (this->currentController != nullptr && !SDL_GamepadConnected(this->currentController)))
-    {
-        this->closeInputDevice();
-        this->openInputDevice(this->currentDevice);
-    }
-
     // process SDL events
     SDL_Event event;
     while (SDL_PeepEvents(&event, 1, SDL_GETEVENT, SDL_EVENT_FIRST, SDL_EVENT_LAST) == 1)
@@ -355,8 +360,8 @@ void MainDialog::on_EventFilter_KeyPressed(QKeyEvent *event)
     SDL_KeyboardEvent keyboardEvent;
     keyboardEvent.down = true;
     keyboardEvent.type = SDL_EVENT_KEY_DOWN;
-    keyboardEvent.scancode = (SDL_Scancode)key;
-    keyboardEvent.key = (SDL_Keycode)key;
+    keyboardEvent.scancode = static_cast<SDL_Scancode>(key);
+    keyboardEvent.key = static_cast<SDL_Keycode>(key);
     keyboardEvent.mod = mod;
 
     SDL_Event sdlEvent;
@@ -374,8 +379,8 @@ void MainDialog::on_EventFilter_KeyReleased(QKeyEvent *event)
     SDL_KeyboardEvent keyboardEvent;
     keyboardEvent.down = false;
     keyboardEvent.type = SDL_EVENT_KEY_UP;
-    keyboardEvent.scancode = (SDL_Scancode)key;
-    keyboardEvent.key = (SDL_Keycode)key;
+    keyboardEvent.scancode = static_cast<SDL_Scancode>(key);
+    keyboardEvent.key = static_cast<SDL_Keycode>(key);
     keyboardEvent.mod = mod;
 
     SDL_Event sdlEvent;
