@@ -10,6 +10,9 @@
 #include "ControllerWidget.hpp"
 #include "UserInterface/OptionsDialog.hpp"
 
+#include "UserInterface/Layout/AlignCenterLayout.hpp"
+
+#include "UserInterface/UICommon.hpp"
 #include "common.hpp"
 
 #include <RMG-Core/RomSettings.hpp>
@@ -25,12 +28,24 @@
 
 #include <SDL3/SDL.h>
 
+using namespace UserInterface::Layout;
 using namespace UserInterface::Widget;
 
 ControllerWidget::ControllerWidget(QWidget* parent, EventFilter* eventFilter) : QWidget(parent)
 {
     this->setupUi(this);
 
+    // sadly Qt Designer doesn't support inserting
+    // custom layouts, so just replace an existing
+    // one with our custom one
+    this->mainLayout->removeItem(this->horizontalLayout_4);
+    delete this->horizontalLayout_4;
+    AlignCenterLayout* layout = new AlignCenterLayout(this->mainLayout->spacing());
+    layout->addWidget(profileGroupBox);
+    layout->addWidget(inputDeviceGroupBox);
+    layout->addWidget(deadZoneGroupBox);
+    this->mainLayout->insertLayout(0, layout);
+
     // install eventFilter to all children and parents
     this->installEventFilter(eventFilter);
     for (auto& widget : this->children())
@@ -175,7 +190,6 @@ ControllerWidget::ControllerWidget(QWidget* parent, EventFilter* eventFilter) :
 
 ControllerWidget::~ControllerWidget()
 {
-
 }
 
 void ControllerWidget::initializeMappingButtons()
@@ -238,6 +252,7 @@ void ControllerWidget::initializeProfileButtons()
 void ControllerWidget::initializeMiscButtons()
 {
     this->inputDeviceRefreshButton->setIcon(QIcon::fromTheme("refresh-line"));
+    this->autoConfigButton->setIcon(QIcon::fromTheme("magic-line"));
     this->resetButton->setIcon(QIcon::fromTheme("restart-line"));
     this->optionsButton->setIcon(QIcon::fromTheme("settings-3-line"));
     this->hotkeysButton->setIcon(QIcon::fromTheme("gamepad-line"));
@@ -245,10 +260,10 @@ void ControllerWidget::initializeMiscButtons()
 
 bool ControllerWidget::isCurrentDeviceKeyboard()
 {
-    const InputDevice device = this->inputDeviceComboBox->currentData().value<InputDevice>();
+    const inputDeviceData deviceData = this->inputDeviceComboBox->currentData().value<inputDeviceData>();
 
-    return device.type == InputDeviceType::Keyboard ||
-            (this->allowKeyboardForAutomatic && device.type == InputDeviceType::Automatic);
+    return deviceData.device.type == InputDeviceType::Keyboard ||
+            (this->allowKeyboardForAutomatic && deviceData.device.type == InputDeviceType::Automatic);
 }
 
 bool ControllerWidget::isCurrentDeviceNotFound()
@@ -266,7 +281,12 @@ void ControllerWidget::disableAllChildren()
             continue;
         }
 
-        QWidget* widget = (QWidget*)object;
+        QWidget* widget = static_cast<QWidget*>(object);
+        if (widget == this->autoConfigButton)
+        { // save the state because whether it's enabled
+          // is set elsewhere
+            this->autoConfigButtonEnabled = widget->isEnabled();
+        }
         if (widget->isEnabled())
         {
             widget->setEnabled(false);
@@ -284,7 +304,11 @@ void ControllerWidget::enableAllChildren()
         }
 
         QWidget* widget = static_cast<QWidget*>(object);
-        if (!widget->isEnabled())
+        if (widget == this->autoConfigButton)
+        { // we have to restore the state
+            widget->setEnabled(this->autoConfigButtonEnabled);
+        }
+        else if (!widget->isEnabled())
         {
             widget->setEnabled(true);
         }
@@ -488,6 +512,22 @@ bool ControllerWidget::hasAnySettingChanged(QString sectionQString)
     return false;
 }
 
+void ControllerWidget::addProfile(const QString& profile, const QString& section)
+{
+    int index = this->profileComboBox->findData(section);
+    if (index == -1)
+    {
+        this->profileComboBox->addItem(profile, section);
+
+        // add tooltip to UI
+        index = this->profileComboBox->findData(section);
+        if (index != -1)
+        {
+            this->profileComboBox->setItemData(index, profile, Qt::ToolTipRole);
+        }
+    }
+}
+
 void ControllerWidget::showErrorMessage(QString text, QString details)
 {
     QMessageBox msgBox(this);
@@ -499,7 +539,7 @@ void ControllerWidget::showErrorMessage(QString text, QString details)
     msgBox.exec();
 }
 
-void ControllerWidget::AddInputDevice(const InputDevice& device)
+void ControllerWidget::AddInputDevice(const InputDevice& device, const InputProfileDBEntry& inputProfile)
 {
     QString deviceName = QString::fromStdString(device.name);
     QString name = deviceName;
@@ -511,18 +551,26 @@ void ControllerWidget::AddInputDevice(const InputDevice& device)
         name += ")";
     }
 
-    this->inputDeviceNameList.append(deviceName);
-    this->inputDeviceComboBox->addItem(name, QVariant::fromValue<InputDevice>(device));
+    this->inputDeviceComboBox->addItem(name, QVariant::fromValue<inputDeviceData>({ device, inputProfile }));
+
+    // add tooltip to UI
+    int index = this->inputDeviceComboBox->findText(name);
+    if (index != -1)
+    {
+        this->inputDeviceComboBox->setItemData(index, name, Qt::ToolTipRole);
+    }
 }
 
 void ControllerWidget::RemoveInputDevice(const InputDevice& device)
 {
-    inputDeviceNameList.removeOne(QString::fromStdString(device.name));
-
-    int index = this->inputDeviceComboBox->findData(QVariant::fromValue<InputDevice>(device));
-    if (index >= 0)
+    inputDeviceData deviceData;
+    for (int i = 0; i < this->inputDeviceComboBox->count(); i++)
     {
-        this->inputDeviceComboBox->removeItem(index);
+        deviceData = this->inputDeviceComboBox->itemData(i).value<inputDeviceData>();
+        if (device == deviceData.device)
+        {
+            this->inputDeviceComboBox->removeItem(i);
+        }
     }
 }
 
@@ -599,7 +647,6 @@ void ControllerWidget::CheckInputDeviceSettings(QString sectionQString)
     int notFoundIndex = this->inputDeviceComboBox->findText("(not found)", Qt::MatchFlag::MatchEndsWith);
     if (notFoundIndex != -1)
     {
-        this->inputDeviceNameList.removeAt(notFoundIndex);
         this->inputDeviceComboBox->removeItem(notFoundIndex);
     }
 
@@ -608,7 +655,8 @@ void ControllerWidget::CheckInputDeviceSettings(QString sectionQString)
 
     for (int i = 0; i < this->inputDeviceComboBox->count(); i++)
     {
-        InputDevice otherDevice = this->inputDeviceComboBox->itemData(i).value<InputDevice>();
+        inputDeviceData otherDeviceData = this->inputDeviceComboBox->itemData(i).value<inputDeviceData>();
+        InputDevice otherDevice = otherDeviceData.device;
         if (device.name == otherDevice.name &&
             device.serial == otherDevice.serial)
         {
@@ -643,9 +691,8 @@ void ControllerWidget::CheckInputDeviceSettings(QString sectionQString)
     { // no match
         QString title = QString::fromStdString(deviceName);
         title += " (not found)";
-        this->inputDeviceNameList.append(QString::fromStdString(deviceName));
-        this->inputDeviceComboBox->addItem(title, QVariant::fromValue<InputDevice>(device));
-        this->inputDeviceComboBox->setCurrentIndex(this->inputDeviceNameList.count() - 1);
+        this->inputDeviceComboBox->addItem(title, QVariant::fromValue<inputDeviceData>({ device, {} }));
+        this->inputDeviceComboBox->setCurrentIndex(this->inputDeviceComboBox->count() - 1);
     }
 }
 
@@ -669,7 +716,7 @@ void ControllerWidget::GetCurrentInputDevice(InputDevice& device, bool ignoreDev
     }
     else
     {
-        device = this->inputDeviceComboBox->itemData(currentIndex).value<InputDevice>();
+        device = this->inputDeviceComboBox->itemData(currentIndex).value<inputDeviceData>().device;
     }
 }
 
@@ -710,6 +757,9 @@ void ControllerWidget::on_profileComboBox_currentIndexChanged(int value)
     // update previous index
     this->previousProfileComboBoxIndex = value;
 
+    // update tooltip
+    this->profileComboBox->setToolTip(this->profileComboBox->itemText(value));
+
     // reload settings from section
     this->LoadSettings(this->getCurrentSettingsSection());
     // reload input device settings
@@ -724,20 +774,26 @@ void ControllerWidget::on_inputDeviceComboBox_currentIndexChanged(int value)
         return;
     }
 
-    InputDevice device = this->inputDeviceComboBox->itemData(value).value<InputDevice>();
+    inputDeviceData deviceData = this->inputDeviceComboBox->itemData(value).value<inputDeviceData>();
 
     this->ClearControllerImage();
 
     if (this->isCurrentDeviceNotFound())
     {
-        device = { };
+        deviceData.device = { };
     }
 
-    // set plugged in state
-    this->setPluggedIn(device.type != InputDeviceType::None &&
-                       device.type != InputDeviceType::EmulateVRU);
+    // set whether auto configure button is enabled
+    this->autoConfigButton->setEnabled(deviceData.inputProfile.valid);
 
-    emit this->CurrentInputDeviceChanged(this, device);
+    // set plugged in state
+    this->setPluggedIn(deviceData.device.type != InputDeviceType::None &&
+                       deviceData.device.type != InputDeviceType::EmulateVRU);
+
+    // update tooltip
+    this->inputDeviceComboBox->setToolTip(this->inputDeviceComboBox->itemText(value));
+
+    emit this->CurrentInputDeviceChanged(this, deviceData.device);
 }
 
 void ControllerWidget::on_inputDeviceRefreshButton_clicked()
@@ -785,7 +841,7 @@ void ControllerWidget::on_addProfileButton_clicked()
     section = this->getUserProfileSectionName(newProfile);
 
     // add profile to UI
-    this->profileComboBox->addItem(newProfile, section);
+    this->addProfile(newProfile, section);
     this->profileComboBox->setCurrentText(newProfile);
 
     // add profile to settings
@@ -869,6 +925,57 @@ void ControllerWidget::on_removeProfileButton_clicked()
     }
 }
 
+void ControllerWidget::on_autoConfigButton_clicked()
+{
+    inputDeviceData deviceData = this->inputDeviceComboBox->currentData().value<inputDeviceData>();
+    QString name;
+
+    for (const auto& inputEntry : deviceData.inputProfile.inputEntries)
+    {
+        if (inputEntry.button != N64ControllerButton::Invalid)
+        {
+            for (const auto& buttonMapping : this->buttonWidgetMappings)
+            {
+                if (buttonMapping.button == inputEntry.button)
+                {
+                    buttonMapping.buttonWidget->Clear();
+                    for (int i = 0; i < inputEntry.data.size(); i++)
+                    {
+                        if (inputEntry.inputTypes[i] == InputType::GamepadButton)
+                        {
+                            name = UICommon::GetGamepadButtonText(this->currentGamepad, static_cast<SDL_GamepadButton>(inputEntry.data[i]));
+                        }
+                        else
+                        {
+                            name = inputEntry.names[i];
+                        }
+
+                        buttonMapping.buttonWidget->AddInputData(inputEntry.inputTypes[i], inputEntry.data[i], 
+                                                                 inputEntry.extraData[i], name);
+                    }
+                    break;
+                }
+            }
+        }
+        else if (inputEntry.axis != InputAxisDirection::Invalid)
+        {
+            for (const auto& axisMapping : this->joystickWidgetMappings)
+            {
+                if (axisMapping.direction == inputEntry.axis)
+                {
+                    axisMapping.buttonWidget->Clear();
+                    for (int i = 0; i < inputEntry.data.size(); i++)
+                    {
+                        axisMapping.buttonWidget->AddInputData(inputEntry.inputTypes[i], inputEntry.data[i], 
+                                                                 inputEntry.extraData[i], inputEntry.names[i]);
+                    }
+                    break;
+                }
+            }
+        }
+    }
+}
+
 void ControllerWidget::on_resetButton_clicked()
 {
     const QString section = this->getCurrentSettingsSection();
@@ -885,12 +992,12 @@ void ControllerWidget::on_resetButton_clicked()
 
 void ControllerWidget::on_optionsButton_clicked()
 {
-    InputDevice device = this->inputDeviceComboBox->currentData().value<InputDevice>();
-    const bool isKeyboard = device.type == InputDeviceType::Keyboard;
+    inputDeviceData deviceData = this->inputDeviceComboBox->currentData().value<inputDeviceData>();
+    const bool isKeyboard = deviceData.device.type == InputDeviceType::Keyboard;
 
     OptionsDialog dialog(this, this->optionsDialogSettings,
                          isKeyboard ? nullptr : this->currentJoystick, 
-                         isKeyboard ? nullptr : this->currentController);
+                         isKeyboard ? nullptr : this->currentGamepad);
     int ret = dialog.exec();
 
     // when saved, store settings for later
@@ -903,8 +1010,7 @@ void ControllerWidget::on_optionsButton_clicked()
 void ControllerWidget::on_hotkeysButton_clicked()
 {
     HotkeysDialog dialog(this, this->hotkeySettingMappings,
-                         this->isCurrentJoystickGameController,
-                         this->currentJoystickId,
+                         this->currentJoystickId, this->currentGamepad,
                          this->optionsDialogSettings.FilterEventsForButtons,
                          this->optionsDialogSettings.RemoveDuplicateMappings);
 
@@ -1039,7 +1145,7 @@ void ControllerWidget::on_MainDialog_SdlEvent(SDL_Event* event)
             if ((event->type == SDL_EVENT_GAMEPAD_BUTTON_DOWN) ||
                 (event->type == SDL_EVENT_GAMEPAD_BUTTON_UP))
             { // gamepad button
-                if (!this->isCurrentJoystickGameController &&
+                if (!this->isCurrentJoystickGamepad &&
                     this->optionsDialogSettings.FilterEventsForButtons)
                 {
                     return;
@@ -1049,12 +1155,12 @@ void ControllerWidget::on_MainDialog_SdlEvent(SDL_Event* event)
                 inputType = InputType::GamepadButton;
                 sdlButton = event->gbutton.button;
                 sdlButtonPressed = (event->type == SDL_EVENT_GAMEPAD_BUTTON_DOWN);
-                sdlButtonName = SDL_GetGamepadStringForButton((SDL_GamepadButton)sdlButton);
+                sdlButtonName = UICommon::GetGamepadButtonText(this->currentGamepad, static_cast<SDL_GamepadButton>(sdlButton));
             }
             else if ((event->type == SDL_EVENT_JOYSTICK_BUTTON_DOWN) ||
                      (event->type == SDL_EVENT_JOYSTICK_BUTTON_UP))
             { // joystick button
-                if (this->isCurrentJoystickGameController &&
+                if (this->isCurrentJoystickGamepad &&
                     this->optionsDialogSettings.FilterEventsForButtons)
                 {
                     return;
@@ -1154,7 +1260,7 @@ void ControllerWidget::on_MainDialog_SdlEvent(SDL_Event* event)
             bool sdlHatCentered = (sdlHatDirection == SDL_HAT_CENTERED);
             QString sdlHatName  = "hat " + QString::number(sdlHat) + ":" + QString::number(sdlHatDirection);
 
-            if (this->isCurrentJoystickGameController &&
+            if (this->isCurrentJoystickGamepad &&
                 this->optionsDialogSettings.FilterEventsForButtons)
             {
                 return;
@@ -1293,7 +1399,7 @@ void ControllerWidget::on_MainDialog_SdlEvent(SDL_Event* event)
 
             if (event->type == SDL_EVENT_GAMEPAD_AXIS_MOTION)
             { // gamepad axis
-                if (!this->isCurrentJoystickGameController &&
+                if (!this->isCurrentJoystickGamepad &&
                     this->optionsDialogSettings.FilterEventsForAxis)
                 {
                     return;
@@ -1303,12 +1409,12 @@ void ControllerWidget::on_MainDialog_SdlEvent(SDL_Event* event)
                 inputType = InputType::GamepadAxis;
                 sdlAxis = event->gaxis.axis;
                 sdlAxisValue = event->gaxis.value;
-                sdlAxisName = SDL_GetGamepadStringForAxis((SDL_GamepadAxis)sdlAxis);
+                sdlAxisName = SDL_GetGamepadStringForAxis(static_cast<SDL_GamepadAxis>(sdlAxis));
                 sdlAxisName += sdlAxisValue > 0 ? "+" : "-";
             }
             else
             { // joystick axis
-                if (this->isCurrentJoystickGameController &&
+                if (this->isCurrentJoystickGamepad &&
                     this->optionsDialogSettings.FilterEventsForAxis)
                 {
                     return;
@@ -1419,7 +1525,7 @@ void ControllerWidget::on_MainDialog_SdlEvent(SDL_Event* event)
                 break;
             }
 
-            const SDL_Scancode sdlButton = (SDL_Scancode)event->key.scancode;
+            const SDL_Scancode sdlButton = static_cast<SDL_Scancode>(event->key.scancode);
             const bool sdlButtonPressed = (event->type == SDL_EVENT_KEY_DOWN);
 
             // handle button widget
@@ -1508,8 +1614,8 @@ void ControllerWidget::on_MainDialog_SdlEventPollFinished()
 
 bool ControllerWidget::IsPluggedIn()
 {
-    const InputDevice device = this->inputDeviceComboBox->currentData().value<InputDevice>();
-    return device.type != InputDeviceType::None;
+    const inputDeviceData deviceData = this->inputDeviceComboBox->currentData().value<inputDeviceData>();
+    return deviceData.device.type != InputDeviceType::None;
 }
 
 void ControllerWidget::SetOnlyLoadGameProfile(bool value, CoreRomHeader romHeader, CoreRomSettings romSettings)
@@ -1532,7 +1638,7 @@ void ControllerWidget::SetSettingsSection(QString profile, QString section)
     this->settingsSection = section;
     if (!this->onlyLoadGameProfile)
     {
-        this->profileComboBox->addItem(profile, section);
+        this->addProfile(profile, section);
     }
 }
 
@@ -1570,14 +1676,18 @@ void ControllerWidget::LoadSettings()
         this->gameSection = section + " Game " + QString::fromStdString(romSettings.MD5);
 
         // use internal rom name
-        QString internalName = QString::fromStdString(romHeader.Name);
+        QString name = QString::fromStdString(romHeader.Name);
+        if (name.isEmpty())
+        { // fallback to partial MD5 hash name
+            name = QString::fromStdString(romSettings.MD5).left(10);
+        }
 
         // add game specific profile when 
         // it doesn't exist yet
         int index = this->profileComboBox->findData(this->gameSection);
         if (index == -1)
         {
-            this->profileComboBox->addItem(internalName, this->gameSection);
+            this->addProfile(name, this->gameSection);
         }
 
         // if a game specific section exists,
@@ -1594,7 +1704,7 @@ void ControllerWidget::LoadSettings()
                 !CoreSettingsKeyExists(this->gameSection.toStdString(), "UseGameProfile") ||
                 CoreSettingsGetBoolValue(SettingsID::Input_UseGameProfile, this->gameSection.toStdString()))
             {
-                this->profileComboBox->setCurrentText(internalName);
+                this->profileComboBox->setCurrentText(name);
 
                 // only use game section if it exists
                 if (CoreSettingsSectionExists(this->gameSection.toStdString()))
@@ -1628,7 +1738,7 @@ void ControllerWidget::LoadSettings()
                 int index = this->profileComboBox->findData(profileSection);
                 if (index == -1)
                 {
-                    this->profileComboBox->addItem(QString::fromStdString(profile), profileSection);
+                    this->addProfile(QString::fromStdString(profile), profileSection);
                 }
             }
         }
@@ -1714,7 +1824,7 @@ void ControllerWidget::LoadSettings(QString sectionQString, bool loadUserProfile
         // add all input data
         for (int i = 0; i < count; i++)
         {
-            buttonSetting.button->AddInputData((InputType)types[i], data[i], extraData[i], QString::fromStdString(names.at(i)));
+            buttonSetting.button->AddInputData(static_cast<InputType>(types[i]), data[i], extraData[i], QString::fromStdString(names.at(i)));
         }
     }
 
@@ -1939,20 +2049,25 @@ void ControllerWidget::RevertSettings()
     }
 }
 
-void ControllerWidget::SetCurrentJoystickID(SDL_JoystickID joystickId)
+void ControllerWidget::SetCurrentInputDevice(SDL_JoystickID joystickId, SDL_Joystick* joystick, 
+                                             SDL_Gamepad* gamepad, const InputProfileDBEntry& inputProfile)
 {
     this->currentJoystickId = joystickId;
-}
+    this->currentJoystick = joystick;
+    this->currentGamepad = gamepad;
+    this->isCurrentJoystickGamepad = (gamepad != nullptr);
 
-void ControllerWidget::SetIsCurrentJoystickGameController(bool isGameController)
-{
-    this->isCurrentJoystickGameController = isGameController;
-}
+    // we can enable the auto-configure button when we have a gamepad
+    this->autoConfigButton->setEnabled(gamepad != nullptr);
 
-void ControllerWidget::SetCurrentJoystick(SDL_Joystick* joystick, SDL_Gamepad* controller)
-{
-    this->currentJoystick   = joystick;
-    this->currentController = controller;
+    // update input profile when needed
+    if (inputProfile.valid)
+    {
+        int currentIndex = this->inputDeviceComboBox->currentIndex();
+        inputDeviceData deviceData = this->inputDeviceComboBox->itemData(currentIndex).value<inputDeviceData>();
+        deviceData.inputProfile = inputProfile;
+        this->inputDeviceComboBox->setItemData(currentIndex, QVariant::fromValue<inputDeviceData>(deviceData));
+    }
 }
 
 void ControllerWidget::AddUserProfile(QString name, QString section)
@@ -1960,7 +2075,7 @@ void ControllerWidget::AddUserProfile(QString name, QString section)
     int index = this->profileComboBox->findData(section);
     if (index == -1)
     {
-        this->profileComboBox->addItem(name, section);
+        this->addProfile(name, section);
     }
 }
 
