@@ -16,6 +16,7 @@
 #include "UserInterface/OptionsDialog.hpp"
 #include "UserInterface/HotkeysDialog.hpp"
 #include "UserInterface/EventFilter.hpp"
+#include "Utilities/InputProfileDB.hpp"
 
 using namespace UserInterface::Widget;
 
@@ -43,7 +44,6 @@ private:
 
     OptionsDialogSettings optionsDialogSettings;
 
-    QList<QString> inputDeviceNameList;
     MappingButton* currentButton = nullptr;
     bool addMappingToButton      = false;
 
@@ -51,6 +51,8 @@ private:
     QList<QString> removedProfiles;
     QList<QString> addedProfiles;
 
+    bool autoConfigButtonEnabled = false;
+
     struct buttonWidgetMapping
     {
         enum N64ControllerButton button;
@@ -72,6 +74,12 @@ private:
         SettingsID extraDataSettingsId;
     };
 
+    struct inputDeviceData
+    {
+        InputDevice device;
+        InputProfileDBEntry inputProfile;
+    };
+
     QList<buttonWidgetMapping> buttonWidgetMappings;
     QList<axisWidgetMapping> joystickWidgetMappings;
     QList<buttonSettingMapping> buttonSettingMappings;
@@ -101,13 +109,15 @@ private:
 
     bool hasAnySettingChanged(QString section);
 
+    void addProfile(const QString& profile, const QString& section);
+
     void showErrorMessage(QString text, QString details = "");
 
-    SDL_JoystickID currentJoystickId     = -1;
-    bool isCurrentJoystickGameController = false;
+    SDL_JoystickID currentJoystickId = -1;
+    bool isCurrentJoystickGamepad = false;
 
-    SDL_Joystick* currentJoystick         = nullptr;
-    SDL_Gamepad* currentController = nullptr;
+    SDL_Joystick* currentJoystick = nullptr;
+    SDL_Gamepad* currentGamepad = nullptr;
 
     int previousProfileComboBoxIndex = -1;
 
@@ -123,7 +133,7 @@ public:
     ControllerWidget(QWidget* parent, EventFilter* eventFilter);
     ~ControllerWidget();
 
-    void AddInputDevice(const InputDevice& device);
+    void AddInputDevice(const InputDevice& device, const InputProfileDBEntry& inputProfile);
     void RemoveInputDevice(const InputDevice& device);
     void CheckInputDeviceSettings();
     void CheckInputDeviceSettings(QString section);
@@ -151,9 +161,8 @@ public:
 
     void RevertSettings();
 
-    void SetCurrentJoystickID(SDL_JoystickID joystickId);
-    void SetIsCurrentJoystickGameController(bool isGameController);
-    void SetCurrentJoystick(SDL_Joystick* joystick, SDL_Gamepad* controller);
+    void SetCurrentInputDevice(SDL_JoystickID joystickId, SDL_Joystick* joystick, 
+                               SDL_Gamepad* gamepad, const InputProfileDBEntry& inputProfile);
 
     void AddUserProfile(QString name, QString section);
     void RemoveUserProfile(QString name, QString section);
@@ -170,6 +179,7 @@ private slots:
     void on_addProfileButton_clicked();
     void on_removeProfileButton_clicked();
 
+    void on_autoConfigButton_clicked();
     void on_resetButton_clicked();
     void on_optionsButton_clicked();
     void on_hotkeysButton_clicked();
