@@ -20,6 +20,7 @@
 #include <QColorDialog>
 #include <QDirIterator>
 #include <QLabel>
+#include <QDir>
 
 #include <RMG-Core/CachedRomHeaderAndSettings.hpp>
 #include <RMG-Core/Directories.hpp>
@@ -109,7 +110,7 @@ SettingsDialog::SettingsDialog(QWidget *parent, QString file) : QDialog(parent)
 
     pluginList = CoreGetAllPlugins();
 
-    for (int i = 0; i < (int)SettingsDialogTab::Invalid; i++)
+    for (int i = 0; i < static_cast<int>(SettingsDialogTab::Invalid); i++)
     {
         this->loadSettings(i);
     }
@@ -117,16 +118,8 @@ SettingsDialog::SettingsDialog(QWidget *parent, QString file) : QDialog(parent)
     // connect hotkey settings to slot
     this->commonHotkeySettings(SettingsDialogAction::ConnectSignals);
 
-    // set icons for certain buttons
-    this->clearJapaneseIPLRomPathButton->setText("");
-    this->clearJapaneseIPLRomPathButton->setIcon(QIcon::fromTheme("delete-bin-line"));
-    this->clearAmericanIPLRomPathButton->setText("");
-    this->clearAmericanIPLRomPathButton->setIcon(QIcon::fromTheme("delete-bin-line"));
-    this->clearDevelopmentIPLRomPathButton->setText("");
-    this->clearDevelopmentIPLRomPathButton->setIcon(QIcon::fromTheme("delete-bin-line"));
-
 #ifndef NETPLAY
-    this->innerInterfaceTabWidget->setTabVisible((int)SettingsDialogTab::InterfaceNetplay, false);
+    this->innerInterfaceTabWidget->setTabVisible(static_cast<int>(SettingsDialogTab::InterfaceNetplay), false);
 #endif // !NETPLAY
 
 #ifndef UPDATER
@@ -172,12 +165,12 @@ int SettingsDialog::currentIndex(void)
 void SettingsDialog::restoreDefaults(int stackedWidgetIndex)
 {
     if (stackedWidgetIndex < 0 || 
-        stackedWidgetIndex > (int)SettingsDialogTab::Invalid)
+        stackedWidgetIndex > static_cast<int>(SettingsDialogTab::Invalid))
     {
         return;
     }
 
-    switch ((SettingsDialogTab)stackedWidgetIndex)
+    switch (static_cast<SettingsDialogTab>(stackedWidgetIndex))
     {
     default:
         break;
@@ -229,12 +222,12 @@ void SettingsDialog::restoreDefaults(int stackedWidgetIndex)
 void SettingsDialog::loadSettings(int stackedWidgetIndex)
 {
     if (stackedWidgetIndex < 0 || 
-        stackedWidgetIndex > (int)SettingsDialogTab::Invalid)
+        stackedWidgetIndex > static_cast<int>(SettingsDialogTab::Invalid))
     {
         return;
     }
 
-    switch ((SettingsDialogTab)stackedWidgetIndex)
+    switch (static_cast<SettingsDialogTab>(stackedWidgetIndex))
     {
     default:
         break;
@@ -377,7 +370,7 @@ void SettingsDialog::loadGamePluginSettings(void)
 
     for (const auto &p : this->pluginList)
     {
-        index = ((int)p.Type - 1);
+        index = (static_cast<int>(p.Type) - 1);
 
         comboBox = comboBoxArray[index];
         comboBox->addItem(QString::fromStdString(p.Name), QString::fromStdString(p.File));
@@ -418,16 +411,10 @@ void SettingsDialog::loadDirectorySettings(void)
     const std::string screenshotDir = CoreSettingsGetStringValue(SettingsID::Core_ScreenshotPath);
     const std::string saveStateDir = CoreSettingsGetStringValue(SettingsID::Core_SaveStatePath);
     const std::string saveSramDir = CoreSettingsGetStringValue(SettingsID::Core_SaveSRAMPath);
-    const bool overrideUserDirs = CoreSettingsGetBoolValue(SettingsID::Core_OverrideUserDirs);
-    const std::string userDataDir = CoreSettingsGetStringValue(SettingsID::Core_UserDataDirOverride);
-    const std::string userCacheDir = CoreSettingsGetStringValue(SettingsID::Core_UserCacheDirOverride);
 
     this->screenshotDirLineEdit->setText(QString::fromStdString(screenshotDir));
     this->saveStateDirLineEdit->setText(QString::fromStdString(saveStateDir));
     this->saveSramDirLineEdit->setText(QString::fromStdString(saveSramDir));
-    this->overrideUserDirsGroupBox->setChecked(overrideUserDirs);
-    this->userDataDirLineEdit->setText(QString::fromStdString(userDataDir));
-    this->userCacheDirLineEdit->setText(QString::fromStdString(userCacheDir));
 }
 
 void SettingsDialog::load64DDSettings(void)
@@ -515,7 +502,8 @@ void SettingsDialog::loadInterfaceOSDSettings(void)
     std::vector<int> textColor = CoreSettingsGetIntListValue(SettingsID::GUI_OnScreenDisplayTextColor);
     if (backgroundColor.size() == 4)
     {
-        this->currentBackgroundColor = QColor(backgroundColor.at(0), backgroundColor.at(1), backgroundColor.at(2), backgroundColor.at(3));
+        this->currentBackgroundColor = QColor(backgroundColor.at(0), backgroundColor.at(1), 
+                                              backgroundColor.at(2), backgroundColor.at(3));
         this->chooseColor(this->changeBackgroundColorButton, &this->currentBackgroundColor, true);
     }
     if (textColor.size() == 4)
@@ -617,9 +605,6 @@ void SettingsDialog::loadDefaultDirectorySettings(void)
     this->screenshotDirLineEdit->setText(QString::fromStdString(CoreSettingsGetDefaultStringValue(SettingsID::Core_ScreenshotPath)));
     this->saveStateDirLineEdit->setText(QString::fromStdString(CoreSettingsGetDefaultStringValue(SettingsID::Core_SaveStatePath)));
     this->saveSramDirLineEdit->setText(QString::fromStdString(CoreSettingsGetDefaultStringValue(SettingsID::Core_SaveSRAMPath)));
-    this->overrideUserDirsGroupBox->setChecked(CoreSettingsGetDefaultBoolValue(SettingsID::Core_OverrideUserDirs));
-    this->userDataDirLineEdit->setText(QString::fromStdString(CoreSettingsGetDefaultStringValue(SettingsID::Core_UserDataDirOverride)));
-    this->userCacheDirLineEdit->setText(QString::fromStdString(CoreSettingsGetDefaultStringValue(SettingsID::Core_UserCacheDirOverride)));
 }
 
 void SettingsDialog::loadDefault64DDSettings(void)
@@ -680,7 +665,8 @@ void SettingsDialog::loadDefaultInterfaceOSDSettings(void)
     const std::vector<int> textColor = CoreSettingsGetDefaultIntListValue(SettingsID::GUI_OnScreenDisplayTextColor);
     if (backgroundColor.size() == 4)
     {
-        this->currentBackgroundColor = QColor(backgroundColor.at(0), backgroundColor.at(1), backgroundColor.at(2), backgroundColor.at(3));
+        this->currentBackgroundColor = QColor(backgroundColor.at(0), backgroundColor.at(1), 
+                                              backgroundColor.at(2), backgroundColor.at(3));
         this->chooseColor(this->changeBackgroundColorButton, &this->currentBackgroundColor, true);
     }
     if (textColor.size() == 4)
@@ -829,7 +815,7 @@ void SettingsDialog::saveGamePluginSettings(void)
         comboBox = comboBoxArray[i];
         id = settingsIdArray[i];
 
-        if (comboBox->currentIndex() != 0 )
+        if (comboBox->currentIndex() != 0)
         {
             CoreSettingsSetValue(id, this->gameSection, comboBox->currentData().toString().toStdString());
         }
@@ -859,10 +845,6 @@ void SettingsDialog::saveDirectorySettings(void)
     CoreSettingsSetValue(SettingsID::Core_ScreenshotPath, this->screenshotDirLineEdit->text().toStdString());
     CoreSettingsSetValue(SettingsID::Core_SaveStatePath, this->saveStateDirLineEdit->text().toStdString());
     CoreSettingsSetValue(SettingsID::Core_SaveSRAMPath, this->saveSramDirLineEdit->text().toStdString());
-
-    CoreSettingsSetValue(SettingsID::Core_OverrideUserDirs, this->overrideUserDirsGroupBox->isChecked());
-    CoreSettingsSetValue(SettingsID::Core_UserDataDirOverride, this->userDataDirLineEdit->text().toStdString());
-    CoreSettingsSetValue(SettingsID::Core_UserCacheDirOverride, this->userCacheDirLineEdit->text().toStdString());
 }
 
 void SettingsDialog::save64DDSettings(void)
@@ -1129,7 +1111,7 @@ void SettingsDialog::commonPluginSettings(SettingsDialogAction action)
 
     for (const auto &p : this->pluginList)
     {
-        index = ((int)p.Type - 1);
+        index = (static_cast<int>(p.Type) - 1);
         comboBox = comboBoxArray[index];
         pluginFileName = pluginFileNames[index];
         pluginName = QString::fromStdString(p.Name);
@@ -1214,7 +1196,13 @@ void SettingsDialog::hideEmulationInfoText(void)
 
 void SettingsDialog::chooseDirectory(QLineEdit *lineEdit, QString caption)
 {
-    QString dir = QFileDialog::getExistingDirectory(this, caption);
+    QString currentDir = lineEdit->text();
+    if (!QDir(currentDir).exists())
+    {
+        currentDir = "";
+    }
+
+    QString dir = QFileDialog::getExistingDirectory(this, caption, currentDir);
     if (dir.isEmpty())
     {
         return;
@@ -1223,7 +1211,7 @@ void SettingsDialog::chooseDirectory(QLineEdit *lineEdit, QString caption)
     lineEdit->setText(QDir::toNativeSeparators(dir));
 }
 
-void SettingsDialog::chooseFile(QLineEdit *lineEdit, QString caption, QString filter, QString md5)
+void SettingsDialog::chooseFile(QLineEdit *lineEdit, QString caption, QString filter, QStringList md5List)
 {
     QString file = QFileDialog::getOpenFileName(this, caption, "", filter);
     if (file.isEmpty())
@@ -1231,7 +1219,7 @@ void SettingsDialog::chooseFile(QLineEdit *lineEdit, QString caption, QString fi
         return;
     }
 
-    if (!md5.isEmpty())
+    if (!md5List.isEmpty())
     {
         QFile qFile(file);
         if (!qFile.open(QFile::ReadOnly))
@@ -1241,15 +1229,29 @@ void SettingsDialog::chooseFile(QLineEdit *lineEdit, QString caption, QString fi
         }
 
         QCryptographicHash hash(QCryptographicHash::Algorithm::Md5);
-        if (hash.addData(&qFile))
+        if (!hash.addData(&qFile))
         {
-            QString md5Hash = QString(hash.result().toHex());
-            if (md5Hash != md5)
+            QtMessageBox::Error(this, "MD5 hashing failed", "QCryptographicHash::addData() Failed");
+            return;
+        }
+
+        QString md5Hash = QString(hash.result().toHex());
+
+        bool hashFound = false;
+        for (const QString& md5 : md5List)
+        {
+            if (md5Hash == md5)
             {
-                QtMessageBox::Error(this, "MD5 mismatch", "Expected file with MD5: \"" + md5 + "\"");
-                return;
+                hashFound = true;
+                break;
             }
         }
+
+        if (!hashFound)
+        {
+            QtMessageBox::Error(this, "MD5 mismatch", "Expected file with MD5: \"" + md5List.first() + "\"");
+            return;
+        }
     }
 
     lineEdit->setText(QDir::toNativeSeparators(file));
@@ -1375,16 +1377,6 @@ void SettingsDialog::on_changeSaveSramDirButton_clicked(void)
     this->chooseDirectory(this->saveSramDirLineEdit, tr("Select Save (SRAM) Directory"));
 }
 
-void SettingsDialog::on_changeUserDataDirButton_clicked(void)
-{
-    this->chooseDirectory(this->userDataDirLineEdit, tr("Select User Data Directory"));
-}
-
-void SettingsDialog::on_changeUserCacheDirButton_clicked(void)
-{
-    this->chooseDirectory(this->userCacheDirLineEdit, tr("Select User Cache Directory"));
-}
-
 void SettingsDialog::on_changeJapaneseIPLRomPathButton_clicked(void)
 {
     this->chooseFile(this->japaneseIPLRomLineEdit, tr("Open Japanese Retail 64DD IPL"), "IPL ROMs (*.n64)");
@@ -1573,10 +1565,10 @@ void SettingsDialog::on_coreCpuEmulatorComboBox_currentIndexChanged(int index)
 
 void SettingsDialog::on_changeNTSCPifRomButton_clicked(void)
 {
-    this->chooseFile(this->ntscPifRomLineEdit, tr("Open NTSC PIF ROM"), "PIF ROMs (*.rom)", "5c124e7948ada85da603a522782940d0");
+    this->chooseFile(this->ntscPifRomLineEdit, tr("Open NTSC PIF ROM"), "PIF ROMs (*.rom)", { "5C124E7948ADA85DA603A522782940D0", "4921D5F2165DEE6E2496F4388C4C81DA" });
 }
 
 void SettingsDialog::on_changePALPifRomButton_clicked(void)
 {
-    this->chooseFile(this->palPifRomLineEdit, tr("Open PAL PIF ROM"), "PIF ROMs (*.rom)", "d4232dc935cad0650ac2664d52281f3a");
+    this->chooseFile(this->palPifRomLineEdit, tr("Open PAL PIF ROM"), "PIF ROMs (*.rom)", { "D4232DC935CAD0650AC2664D52281F3A", "2B6EEC586FAA43F3462333B844834554" });
 }
