@@ -49,12 +49,18 @@
 #include <QSettings>
 #include <QStatusBar>
 #include <QMenuBar>
-#include <cstdlib>
 #include <QString>
 #include <QTimer>
-#include <cmath>
+#include <QDir>
 #include <QUrl>
 
+#ifdef KCA_DRAG_DROP
+#include <KUrlMimeData>
+#endif
+
+#include <cstdlib>
+#include <cmath>
+
 #include <RMG-Core/CachedRomHeaderAndSettings.hpp>
 #include <RMG-Core/SpeedLimiter.hpp>
 #include <RMG-Core/Directories.hpp>
@@ -1149,23 +1155,19 @@ void MainWindow::configureActions(void)
     for (int i = 0; i < 12; i++)
     {
         QAction* speedAction = speedActions[i];
+        int speedActionNumber = speedActionNumbers[i];
 
         speedAction->setCheckable(true);
-        speedAction->setChecked(currentSpeedFactor == speedActionNumbers[i]);
+        speedAction->setChecked(currentSpeedFactor == speedActionNumber);
         speedAction->setActionGroup(speedActionGroup);
 
         // connect emulation speed action here because we need to do
         // something special for them
-        connect(speedAction, &QAction::triggered, [=, this](bool checked)
+        connect(speedAction, &QAction::triggered, [this, speedActionNumber](bool checked)
         {
             if (checked)
             {
-                QString factorText = speedAction->text().split("%").first();
-                // sometimes the text can contain a '&'
-                // which will make the toInt() function return 0
-                // so strip it out
-                factorText.remove('&');
-                this->on_Action_System_SpeedFactor(factorText.toInt());
+                this->on_Action_System_SpeedFactor(speedActionNumber);
             }
         });
     }
@@ -1182,16 +1184,11 @@ void MainWindow::configureActions(void)
 
         // connect slot action here because we need to do
         // something special for them
-        connect(slotAction, &QAction::triggered, [=, this](bool checked)
+        connect(slotAction, &QAction::triggered, [this, i](bool checked)
         {
             if (checked)
             {
-                QString slotText = slotAction->text().split(" ").at(1);
-                // sometimes the text can contain a '&'
-                // which will make the toInt() function return 0
-                // so strip it out
-                slotText.remove('&');
-                this->on_Action_System_CurrentSaveState(slotText.toInt());
+                this->on_Action_System_CurrentSaveState(i);
             }
         });
     }
@@ -1465,11 +1462,15 @@ void MainWindow::on_EventFilter_KeyReleased(QKeyEvent *event)
 
 void MainWindow::on_EventFilter_FileDropped(QDropEvent *event)
 {
-#ifdef DRAG_DROP
     const QMimeData *mimeData = event->mimeData();
 
-    if (!mimeData->hasUrls() || mimeData->urls().empty() ||
-        !mimeData->urls().first().isLocalFile())
+    QList<QUrl> urls = mimeData->urls();
+#ifdef KCA_DRAG_DROP
+    urls = KUrlMimeData::urlsFromMimeData(mimeData, KUrlMimeData::PreferLocalUrls);
+#endif
+
+    if (!mimeData->hasUrls() || urls.empty() ||
+        !urls.first().isLocalFile())
     {
         return;
     }
@@ -1497,7 +1498,7 @@ void MainWindow::on_EventFilter_FileDropped(QDropEvent *event)
         }
     }
 
-    file = mimeData->urls().first().toLocalFile();
+    file = urls.first().toLocalFile();
 
     if (inEmulation)
     {
@@ -1512,7 +1513,6 @@ void MainWindow::on_EventFilter_FileDropped(QDropEvent *event)
     }
 
     this->launchEmulationThread(file, "", refreshRomList, -1, false, true);
-#endif // DRAG_DROP
 }
 
 void MainWindow::on_QGuiApplication_applicationStateChanged(Qt::ApplicationState state)
@@ -2234,8 +2234,14 @@ void MainWindow::on_RomBrowser_PlayGameWithSlot(QString file, int slot)
 
 void MainWindow::on_RomBrowser_ChangeRomDirectory(void)
 {
-    QString dir = QFileDialog::getExistingDirectory(this, tr("Select ROM Directory"));
-    if (!dir.isEmpty())
+    QString currentDir = QString::fromStdString(CoreSettingsGetStringValue(SettingsID::RomBrowser_Directory));
+    if (!QDir(currentDir).exists())
+    {
+        currentDir = "";
+    }
+
+    QString dir = QFileDialog::getExistingDirectory(this, tr("Select ROM Directory"), currentDir);
+    if (!dir.isEmpty() && currentDir != dir)
     {
         CoreSettingsSetValue(SettingsID::RomBrowser_Directory, dir.toStdString());
         this->ui_Widget_RomBrowser->RefreshRomList();
@@ -2250,16 +2256,17 @@ void MainWindow::on_RomBrowser_RomInformation(QString file)
         this->ui_Widget_RomBrowser->StopRefreshRomList();
     }
 
+    CoreRomType romType;
     CoreRomHeader romHeader;
     CoreRomSettings romSettings;
 
-    if (!CoreGetCachedRomHeaderAndSettings(file.toStdU32String(), nullptr, &romHeader, nullptr, &romSettings))
+    if (!CoreGetCachedRomHeaderAndSettings(file.toStdU32String(), &romType, &romHeader, nullptr, &romSettings))
     {
         this->showErrorMessage("CoreGetCachedRomHeaderAndSettings() Failed", QString::fromStdString(CoreGetError()));
         return;
     }
 
-    Dialog::RomInfoDialog dialog(file, romHeader, romSettings, this);
+    Dialog::RomInfoDialog dialog(this, file, romType, romHeader, romSettings);
     dialog.exec();
 
     if (isRefreshingRomList)
@@ -2297,7 +2304,7 @@ void MainWindow::on_RomBrowser_EditGameInputSettings(QString file)
         this->ui_Widget_RomBrowser->StopRefreshRomList();
     }
 
-    if (!CorePluginsOpenROMConfig(CorePluginType::Input, (void*)this, file.toStdU32String()))
+    if (!CorePluginsOpenROMConfig(CorePluginType::Input, this, file.toStdU32String()))
     {
         this->showErrorMessage("CorePluginsOpenROMConfig() Failed", QString::fromStdString(CoreGetError()));
         return;
@@ -2505,8 +2512,8 @@ void MainWindow::on_VidExt_ResizeWindow(int width, int height)
     // account for HiDPI scaling
     if (this->devicePixelRatio() != 1)
     {
-        width  = (int)std::ceil((double)((double)width  / this->devicePixelRatio()));
-        height = (int)std::ceil((double)((double)height / this->devicePixelRatio()));
+        width  = static_cast<int>(std::ceil(static_cast<double>(static_cast<double>(width)  / this->devicePixelRatio())));
+        height = static_cast<int>(std::ceil(static_cast<double>(static_cast<double>(height) / this->devicePixelRatio())));
     }
 
     if (!this->menuBar()->isHidden())
@@ -2708,7 +2715,7 @@ void MainWindow::on_Core_StateCallback(CoreStateCallbackType type, int value)
         case CoreStateCallbackType::EmulationState:
         {
             // update Pause button
-            this->action_System_Pause->setChecked(value == (int)CoreEmulationState::Paused);
+            this->action_System_Pause->setChecked(value == static_cast<int>(CoreEmulationState::Paused));
             // update OSD state
             if (value == (int)CoreEmulationState::Paused)
             {
