@@ -161,9 +161,6 @@ RomBrowserWidget::RomBrowserWidget(QWidget *parent) : QWidget(parent)
     this->gridViewWidget->setModel(this->gridViewProxyModel);
     this->gridViewWidget->setFlow(QListView::Flow::LeftToRight);
     this->gridViewWidget->setResizeMode(QListView::Adjust);
-#ifndef DRAG_DROP
-    this->gridViewWidget->setMovement(QListView::Static);
-#endif // DRAG_DROP
     this->gridViewWidget->setUniformItemSizes(CoreSettingsGetBoolValue(SettingsID::RomBrowser_GridViewUniformItemSizes));
     this->gridViewWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
     this->gridViewWidget->setViewMode(QListView::ViewMode::IconMode);
@@ -183,10 +180,8 @@ RomBrowserWidget::RomBrowserWidget(QWidget *parent) : QWidget(parent)
     connect(this->gridViewWidget, &Widget::RomBrowserGridViewWidget::ZoomOut, this, &RomBrowserWidget::on_ZoomOut);
     connect(this->gridViewWidget, &Widget::RomBrowserGridViewWidget::FileDropped, this, &RomBrowserWidget::FileDropped);
 
-#ifdef DRAG_DROP
     // configure drag & drop
     this->setAcceptDrops(true);
-#endif // DRAG_DROP
 
     // configure context menu policy
     this->setContextMenuPolicy(Qt::ContextMenuPolicy::CustomContextMenu);
@@ -215,20 +210,35 @@ RomBrowserWidget::RomBrowserWidget(QWidget *parent) : QWidget(parent)
 
     // configure context menu contents
     this->action_PlayGame->setText("Play Game");
+    this->action_PlayGame->setIcon(QIcon::fromTheme("gamepad-line"));
     this->action_PlayGameWith->setText("Play Game with Disk");
+    this->action_PlayGameWith->setIcon(QIcon::fromTheme("hard-drive-line"));
     this->menu_PlayGameWithDisk->menuAction()->setText("Play Game with Disk");
+    this->menu_PlayGameWithDisk->menuAction()->setIcon(QIcon::fromTheme("hard-drive-line"));
     this->menu_PlayGameWithSlot->menuAction()->setText("Play Game with State");
+    this->menu_PlayGameWithSlot->menuAction()->setIcon(QIcon::fromTheme("save-3-line"));
     this->action_RefreshRomList->setText("Refresh ROM List");
+    this->action_RefreshRomList->setIcon(QIcon::fromTheme("refresh-line"));
     this->action_OpenRomDirectory->setText("Open ROM Directory");
+    this->action_OpenRomDirectory->setIcon(QIcon::fromTheme("folder-open-line"));
     this->action_ChangeRomDirectory->setText("Change ROM Directory...");
+    this->action_ChangeRomDirectory->setIcon(QIcon::fromTheme("pencil-line"));
     this->action_RomInformation->setText("ROM Information");
+    this->action_RomInformation->setIcon(QIcon::fromTheme("information-line"));
     this->action_EditGameSettings->setText("Edit Game Settings");
+    this->action_EditGameSettings->setIcon(QIcon::fromTheme("settings-3-line"));
     this->action_EditGameInputSettings->setText("Edit Game Input Settings");
+    this->action_EditGameInputSettings->setIcon(QIcon::fromTheme("gamepad-line"));
     this->action_EditCheats->setText("Edit Cheats");
+    this->action_EditCheats->setIcon(QIcon::fromTheme("code-box-line"));
     this->action_ResetColumnSizes->setText("Reset Column Sizes");
+    this->action_ResetColumnSizes->setIcon(QIcon::fromTheme("restart-line"));
     this->menu_Columns->menuAction()->setText("Show/Hide Columns");
+    this->menu_Columns->menuAction()->setIcon(QIcon::fromTheme("eye-line"));
     this->action_SetCoverImage->setText("Set Cover Image...");
+    this->action_SetCoverImage->setIcon(QIcon::fromTheme("file-line"));
     this->action_RemoveCoverImage->setText("Remove Cover Image");
+    this->action_RemoveCoverImage->setIcon(QIcon::fromTheme("delete-bin-line"));
     connect(this->action_PlayGame, &QAction::triggered, this, &RomBrowserWidget::on_Action_PlayGame);
     connect(this->action_PlayGameWith, &QAction::triggered, this, &RomBrowserWidget::on_Action_PlayGameWith);
     connect(this->menu_PlayGameWithDisk, &QMenu::triggered, this, &RomBrowserWidget::on_Menu_PlayGameWithDisk);
@@ -828,14 +838,9 @@ void RomBrowserWidget::generateStateMenu(void)
             saveStateSlotText += saveStateFileInfo.lastModified().toString("dd/MM/yyyy hh:mm");
 
             QAction* slotAction = this->menu_PlayGameWithSlot->addAction(saveStateSlotText);
-            connect(slotAction, &QAction::triggered, [=, this]()
+            connect(slotAction, &QAction::triggered, [this, i]()
             {
-                QString slotText = slotAction->text().split(" ").at(1);
-                // sometimes the text can contain a '&'
-                // which will make the toInt() function return 0
-                // so strip it out
-                slotText.remove('&');
-                this->on_Action_PlayGameWithSlot(slotText.toInt());
+                this->on_Action_PlayGameWithSlot(i);
             });
         }
     }
@@ -859,7 +864,7 @@ void RomBrowserWidget::on_searchWidget_SearchTextChanged(const QString& text)
 void RomBrowserWidget::on_listViewWidget_sortIndicatorChanged(int logicalIndex, Qt::SortOrder sortOrder)
 {
     CoreSettingsSetValue(SettingsID::RomBrowser_ListViewSortSection, logicalIndex);
-    CoreSettingsSetValue(SettingsID::RomBrowser_ListViewSortOrder, (int)sortOrder);
+    CoreSettingsSetValue(SettingsID::RomBrowser_ListViewSortOrder, static_cast<int>(sortOrder));
 }
 
 void RomBrowserWidget::on_listViewWidget_sectionResized(int logicalIndex, int oldWidth, int newWidth)
@@ -972,7 +977,7 @@ void RomBrowserWidget::on_RomBrowserThread_RomsFound(QList<RomSearcherThreadData
 void RomBrowserWidget::on_RomBrowserThread_Finished(bool canceled)
 {
     // sort data
-    this->listViewProxyModel->sort(this->listViewSortSection, (Qt::SortOrder)this->listViewSortOrder);
+    this->listViewProxyModel->sort(this->listViewSortSection, static_cast<Qt::SortOrder>(this->listViewSortOrder));
     this->gridViewProxyModel->sort(0, Qt::SortOrder::AscendingOrder);
 
     // retrieve column settings
